#pragma once

#include "waylandbridgewindow.h"

#include <QThread>
#include <wayland-client.h>

namespace kdk
{

class WaylandBridge : public QThread
{
    Q_OBJECT
public:
    WaylandBridge(QObject *parent = nullptr);
    ~WaylandBridge() override;

    bool init();

    std::vector<WaylandBridgeWindow *> windows() const;

protected:
    void run() override;

Q_SIGNALS:
    void windowCreated(WaylandBridgeWindow *window);
    void showDesktopChanged(bool state);

private:
    bool isInitialized = false;

    wl_display *m_display = nullptr;
    wl_registry *m_registry = nullptr;
    org_kde_plasma_window_management *m_plasmaWindowManagement = nullptr;
    std::vector<WaylandBridgeWindow *> m_windows;

    static const wl_registry_listener m_registryListener;
    static const org_kde_plasma_window_management_listener m_plasmaWindowManagementListener;
    static void on_registry_global(void *data, struct wl_registry *registry, uint32_t name, const char *interface, uint32_t version);
    static void on_plasma_window_management_show_desktop_changed(void *data, struct org_kde_plasma_window_management *org_kde_plasma_window_management, uint32_t state);
    static void on_plasma_window_management_window(void *data, struct org_kde_plasma_window_management *org_kde_plasma_window_management, uint32_t id);
};

} // namespace kdk
