#pragma once

#include "wayland-plasma-window-management-client-protocol.h"

#include <QIcon>
#include <QObject>
#include <QRect>

#include <QDebug>
#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(WAYLAND_BRIDGE)


namespace kdk
{

class WaylandBridge;

class WaylandBridgeWindow : public QObject
{
    Q_OBJECT

public:
    WaylandBridgeWindow(org_kde_plasma_window *window, uint32_t id, WaylandBridge *bridge);
    ~WaylandBridgeWindow() override;

    uint32_t id() const;
    QString title() const;
    QString appId() const;
    int32_t virtualDesktop() const;
    QRect geometry() const;
    QIcon icon() const;
    uint32_t pid() const;
    bool isActive() const;
    bool isMinimized() const;
    bool isMaximized() const;
    bool isFullscreen() const;
    bool isKeepAbove() const;
    bool isKeepBelow() const;
    bool isSkipTaskbar() const;
    bool isSkipSwitcher() const;
    bool isOnAllDesktops() const;
    bool isShaded() const;

    bool isCloseable() const;
    bool isFullscreenable() const;
    bool isMaximizable() const;
    bool isMinimizable() const;
    bool isMovable() const;
    bool isResizable() const;
    bool isShadable() const;
    bool isVirtualDesktopChangeable() const;

    QStringList virtualDesktops() const;

    // request
    void requestActivate();
    void requestClose();
    void requestToggleKeepAbove();
    void requestToggleMinimized();
    void requestToggleMaximized();

    void setOnAllDesktops();
    void demandAttention() const;

Q_SIGNALS:
    void activeWindowChanged(uint32_t id);
    void windowChanged(uint32_t winfo);
    void windowAdded(uint32_t id);
    void windowRemoved(uint32_t id);
    void currentDesktopChanged();
    void isShowingDesktopChanged();
    void titleChanged(uint32_t id);
    void iconChanged(uint32_t id);
    void activeChanged(uint32_t id);
    void fullscreenChanged(uint32_t id);
    void keepAboveChanged(uint32_t id);
    void keepBelowChanged(uint32_t id);
    void minimizedChanged(uint32_t id);
    void maximizedChanged(uint32_t id);
    void onAllDesktopsChanged(uint32_t id);
    void demandsAttentionChanged(uint32_t id);
    void skipTaskbarChanged(uint32_t id);
    void skipSwitcherChanged(uint32_t id);
    void geometryChanged(uint32_t id);

private:
    org_kde_plasma_window *m_window = nullptr;

    const static org_kde_plasma_window_listener m_waylandBridgeWindowListener;
    static void on_plasma_window_title_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *title);
    static void on_plasma_window_app_id_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *app_id);
    static void on_plasma_window_state_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, uint32_t flags);
    static void on_plasma_window_virtual_desktop_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, int32_t number);
    static void on_plasma_window_themed_icon_name_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *name);
    static void on_plasma_window_unmapped(void *data, struct org_kde_plasma_window *org_kde_plasma_window);
    static void on_plasma_window_initial_state(void *data, struct org_kde_plasma_window *org_kde_plasma_window);
    static void on_plasma_window_parent_window(void *data, struct org_kde_plasma_window *org_kde_plasma_window, struct org_kde_plasma_window *parent);
    static void on_plasma_window_geometry(void *data, struct org_kde_plasma_window *org_kde_plasma_window, int32_t x, int32_t y, uint32_t idth, uint32_t height);
    static void on_plasma_window_icon_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window);
    static void on_plasma_window_pid_changed(void *data, struct org_kde_plasma_window *org_kde_plasma_window, uint32_t pid);
    static void on_plasma_window_virtual_desktop_entered(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *id);
    static void on_plasma_window_virtual_desktop_left(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *is);
    static void on_plasma_window_application_menu(void *data, struct org_kde_plasma_window *org_kde_plasma_window, const char *service, const char *object);

    WaylandBridge *m_waylandBridge = nullptr;
    uint32_t m_id = 0;

    QString m_title;
    QString m_appId;
    uint32_t m_state = 0;
    QStringList m_virtualDesktops;
    int32_t m_virtualDesktop = 0;
    QRect m_geometry;
    QIcon m_icon;
    uint32_t m_pid = 0;
};

} // namespace kdk